<?php
/*

 $Id: host.php,v 1.63 2003/07/21 03:33:06 vollmerk Exp $
 $Source: /data/cvsroot/maint/docs/host.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*!
	@header Host Document
	This document handles host related actions like show_hosts, update_host. It doesn't host any actual functions, it holds if statments.
	All you have to do is pass this file a $action, and the nessacary params.

*/


/* Start House Keeping */

	require("../modules/config.php");

	// make sure they are supposed to be here
	check_session();

	setup_user_environment($action, $zone_id);

/* End House Keeping */


$host = new Host($host_id);

//process any submits that may come our way
$title = "Modifying Host Information";

show_template('header');
?> <font class="header1"><?= $title ?></font> <?


/*!
	@function Update_Host
	@discussion this action updates a host, this is passed from show_host.inc, before it updates the host
		    it does all the needed error checking.
	@param host_id		ID of Host we are updating (referenced in DB)
	@param name 		Host Name
	@param domain_id 	Domain ID (referenced in DB)
	@param ha 		Hardware Address
	@param bandwidth_rate	Tag for ratelimiting hosts, requires external script
	@param workgroup_id	Workgroup ID (referenced in DB) for this host
	@param comment		Information Field, Comment applied to Host History
	@param os		Information Field, Operating System
	@param type		Information Field, Hardware Type
	@param location		Information Field, Location of Device (Physical)
	@param serial 		Information Field, Serial Number of Device
	@param other_id		Information Field, Other ID
	@param purchase_date	Information Field, Purchase Date
	@param po_number	Information Field, PO Number
	@param warranty_date	Information Field, Warranty Date
	@param owning_unit	Information Field, Unit/Dept that owns this device
	@param user_id		Information Field, Username Assoiciated with device
	@param expire		Number of days till we get to delete this host
*/	
if ( $action == 'Update' ) {

	// if they used the admin select
	if ($domain_select == "admin") { $domain_id = $admin_domain_id; }

	// do whatever you have to to update the host
	//   the $conf['error'] object gets updated automatically
	$domain = new Domain($domain_id);

        //force lowercase and clean it up
	$ha = $conf['ui']->clean_hardware_address($ha);


	//do a case insensitive match for a host name change 
	//when creating a host case matters but when updating if 
	//they change the case of the hostname we don't need to 
	//worry about it... let them!
	if (strcasecmp($host->name,$name)) {
		$host->check_hostname($name, $domain, $host_id);
	}

	// this is a special case for people who don't want to
	//   be in DHCP ... some ranges don't want to be there.
	if ( $ha != '000000000000' ) {
		$host->check_hardware_address($ha,$host_id,$range_id);
	}

	// we need to check and see if they switch the domain.
	if ($domain_id != $host->domain->id) {
		$host->check_hostname($name,$domain,$host_id);
	}
	
	// if there is not domain_id, or the host doesn't have access AND they aren't an admin then "No Domain for You!"
	// don't care if they didn't change it though
	if ( (!$domain_id || !$host->zone->has_domain_access($domain_id)) && !$user->is_admin && $domain_id != $host->domain->id) {
		$conf['error']->add_error('domain',"This host can not be put in $domain->name");
	}

	// if no errors are detected update the host
	if ( ! $conf['error']->error_state AND $user->has_access($host->zone->id)) {

		if ($expire) { 	
			$expire = time() + (86400 * $expire);
		}

		// update the host with the new information
		$host->update_name($name);
		$host->update_domain($domain_id);
		$host->update_ha($ha);
		$host->update_os($os);
		$host->update_type($type);
		$host->update_location($location);
		$host->update_serial($serial);
		$host->update_other_id($other_id);
		$host->update_status($status);
		$host->update_bandwidth_rate($bandwidth_rate);
		$host->object_history->update($user->username,$comment);
		$host->update_purchase_date($purchase_date);
		$host->update_po_number($po_number);
		$host->update_warranty_date($warranty_date);
		$host->update_owning_unit($owning_unit);
		$host->update_user_id($user_id);
		if ($update_expire) { $host->update_expire($expire); }

		// if the workgroup_id exists we should update this
		//   otherwise leave it alone
		if ( $workgroup_id ) {
			$host->update_workgroup($workgroup_id);
		}

		// setup vars for displaying this new host
		$action_type = "modified";
		$host = new Host($host_id);
		$child = new Child("host",$host->id,0,$host);
		if ($child->has_children) { $action_type .= "_children"; }
	} //if no errors

	// setup the next action
	$action = 'show_host';

} //if action == update

/*!
	@function Enable_Host
	@discussion enables a host, special case required if you are doing this from a search
	@param host_id 		Required for the host object, runs class function $host->update_stats('enabled')
	@param search 		To determin if you are comming from search, _GET var
*/
elseif ( $action == 'enable_host' ) {
	// update the host
	if ($user->has_access($host->zone->id)) {
		$host->update_status('enabled');
	}
	
	// hack for searching retains the view for us
	if ($search) { 
		echo "<br /><br /><font class=\"error\">Host Re-Enabled</font>";
		$action = "show_host";
		$conf['action'] = "show_host";
	} //if search
	else { 
		$action = "show_hosts";
		$conf['action'] = "show_hosts";
		$user->view->freeze();
	} //else

} //if action == enable_host

/*!
	@function Disable_Host
	@discussion disables a host, special case required if you are doing this from a search
	@param host_id		Required for the host object, runs class function $host->update_stats('disable')
	@param search		To determin if you are comming from search, _GET var
*/
elseif ( $action == "disable_host" ) {
	
	if ($user->has_access($host->zone->id)) {
		$host->update_status('disabled');
	}
	
	//if it's a search'd disable for now let's push them off to show_host (single)
	if ($search) { 
		echo "<br /><br /><font class=\"error\">Host Disable</font>";
		$action = "show_host";
		$conf['action'] = "show_host";
	} // if search
	else {
		$action = "show_hosts";
		$conf['action'] = "show_hosts";
		$user->view->freeze();
	} // else
} //if action == disable_host

/*!
	@function bulk_delete
	@discussion After picking the hosts in a builk select actually delete the hosts.
	@param hosts	Array of host ID's (referenced in DB)
*/
elseif ($action == 'bulk_delete') { 

//	$conf['ui']->display_confirmation("Hosts Delete","The following Hosts have been deleted", $conf['web_path'] ."/index.php");
	foreach ($hosts as $host_id) { 
		$host = new Host($host_id);
		echo $host->hostname . "<br />";

		// double check and make sure they have access
		if ($user->has_access($host->zone->id)) {
			$host->delete();
		}
	
	} //end foreach

} // action == bulk_delete

/*!
	@function bulk_update
	@discussion once they have picked they hosts, update them, does checking on the domain changing.
*/
elseif ($action == "bulk_update") {

	foreach ($hosts as $h) {
		$host = new Host($h);
	
		// if they don't have access deny the sucker and break
		if (!$user->has_access($host->zone->id)) {
			$conf['ui']->show_access_denied();
			break;
		}

		//check if they wanted it updated, if so then update their mojo!
		if ($os_update) {
			$host->update_os($os);
		}
		if ($hardware_type_update) {
			$host->update_type($type);
		}
		if ($location_update) {
			$host->update_location($location);
		}
		if ($serial_update) {
			$host->update_serial($serial);
		}
		if ($otherid_update) {
			$host->update_other_id($other_id);
		}
		if ($user_id_update) { 
			$host->update_user_id($user_id);
		}
		if ($owning_unit_update) {
			$host->update_owning_unit($owning_unit);
		}
		if ($warranty_date_update) {
			$host->update_warranty_date($warranty_date);
		}
		if ($po_number_update) {
			$host->update_po_number($po_number);
		}
		if ($domain_update) {
			//First we need to check and make sure that we won't create a duplicate host by doing this
			$domain->id = $domain_id;
	                $host->check_hostname($host->name,$domain,$host->id);

			if (!$conf['error']->error_state) {
				$host->update_domain($domain_id);
			}
			//If we can't move it to the new domain then go ahead and add it to an array so we can display it later.
			else {
				$bad_hosts[] = $host;
			}
		}

		//Update the history for this object
		$host->object_history->update($user->username,$comment);
	
		$new_hosts[] = $host;

	}

	$hosts = $new_hosts;
	$total_items = count($hosts);
	$user->view->limit = count($hosts)+1;
	require ($conf['prefix'] . "/templates/show_hosts.inc");

	/* If they updated the domain */
	if ($domain_update AND count($bad_hosts)) {

		//Now Throw out any that were Bad little monkeys!
		echo "<p><font class=\"fatalerror\">Host Domain's Not Changed.</font><br />All other updates were made to these hosts. The domain wasn't changed to avoid duplication, please change the name of offending hosts.</p>\n";
	
		$hosts = $bad_hosts;
		$total_items = count($hosts);
		$user->view->limit = count($hosts)+1;
		require ($conf['prefix'] . "/templates/show_hosts.inc");

	} //domain_update

} // action == bulk_update



/*!
	@function move_host_zone
	@discussion This moves the host between zones, make sure they have access to the source and dest zones
	@param $host_id	ID of the Host we are moving
	@param $zone_id ID of the zone to move them to
*/
elseif ($action == "move_host_zone" AND $user->has_access($zone_id) AND $user->has_access($host->zone->id)) {

	// update the zone they are in
	$host->update_zone($zone_id);

	$conf['ui']->display_confirmation("Host Zone Updated","$host->hostname has been moved to ". $host->zone->name, $conf['web_path'] . "/host.php?action=show_host&amp;host_id=" . $host->id);

} // move_host_zone

/*!
	@function move_hosts_zone
	@discussion This moves many hosts between zones, it take an array of hosts, only administrtors can do this
	@param $hosts	Array of hosts that we need to move
	@param $zone_id	The ID of the zone we are going to move the hosts to
*/
elseif ($action == "move_hosts_zone" AND $user->is_admin) {

	$zone = new Zone($zone_id);

	foreach ($hosts as $host) {
		$host = new Host($host);
		$host->update_zone($zone_id);
	}

	$conf['ui']->display_confirmation("All Hosts Zones Updated","All hosts have been moved to "  . $zone->name, $conf['web_path'] . "/admin.php");

} // move_hosts_zone

/*
###################################################
END OF ACTION IF BLOCK
###################################################



###################################################
START OF DISPLAY BLOCK
###################################################
*/


/*!
	@function show_host
	@discussion displays a host.
	@param host_id		Used to create host object
	@param action_type	Read/Edit, should be split out into show and edit
*/
if ( $action == 'show_host' ) {

	// Set the view type if we've got it
	if ($host_view) { $user->host_view = $host_view; }	

	// Show the host
	$conf['ui']->show_host($host, $action_type);

} // action == show_host

/*!
	@function select_bulk_delete
	@discussion show the select for a bulk delete.
*/
elseif ($action == "select_bulk_delete") {
	require ($conf['prefix'] . "/templates/show_bulk_delete.inc");
} // action == select_bulk_delete

/*!
	@function confirm_bulk_delete
	@discussion show a confirmation of the bulk delete before we do it.
	@param hosts	Array of hosts for deletion
*/	
elseif ($action == "confirm_bulk_delete") {
	require ($conf['prefix'] . "/templates/show_bulk_delete.inc");
} // action == confirm_bulk_delete

/*!
	@function view_bulk_delete 
	@discussion view the bulk delete, after selecting, before confirmation
	@param select_type	Type of select from bulk_select, range, domain, hostname or ip range.
	@param range_id		If they picked range, the ID
	@param start_range	Int Value of range start
	@param end_range	Int Value of range end
	@param domain_id	If they picked domain, the ID
	@param hostname		The base hostname they want to search on.
*/
elseif ($action == "view_bulk_delete") {
		
	// get the hosts we need
	$hosts = $host->bulk_get($select_type,$range_id,$start_range,$end_range,$domain_id,$hostname);
	
	// if an error hasn't occured let's display these hosts!
	if (!$conf['error']->error_state) {
		if (count($hosts) > 1) { require ($conf['prefix'] . "/templates/show_bulk_delete.inc"); }
		elseif (count($hosts) == "1") { 
			$host = $hosts[0]; 
			require ($conf['prefix'] . "/templates/show_host.inc"); 
			}
		else {
			echo "<p align=\"center\"><font class=\"fatalerror\">No Hosts Found</font></p>";
			}
		} //check for error state

	//if an error has occured then go back to the select!
	else {
		require ($conf['prefix'] . "/templates/show_bulk_select.inc");
	} //end else

} // action == view_bulk_delete

/*! 
	@function select_bulk_update
	@discussion display the bulk update
*/
elseif ($action == "select_bulk_update") {
	require ($conf['prefix'] . "/templates/show_bulk_update.inc");
} //show_bulk_update

/*!
	@function view_bulk_update
	@discussion just like bulk delete, display the selected hosts, and allow them to pick which ones to update
	@param select_type	Type of select from bulk_select, range, domain, hostname or ip range.
	@param range_id		If they picked range, the ID
	@param start_range	Int Value of range start
	@param end_range	Int Value of range end
	@param domain_id	If they picked domain, the ID
	@param hostname		The base hostname they want to search on.
*/
elseif ($action == "view_bulk_update") {

	//get the hosts we need
	$hosts = $host->bulk_get($select_type,$range_id,$start_range,$end_range,$domain_id,$hostname);

        //if an error hasn't occured let's display these hosts!
        if (!$conf['error']->error_state) {
                if (count($hosts) > 1) { require ($conf['prefix'] . "/templates/show_bulk_update.inc"); }
                elseif (count($hosts) == "1") {
                        $host = $hosts[0];
                        require ($conf['prefix'] . "/templates/show_host.inc");
                        }
                else {
                        echo "<p align=\"center\"><font class=\"fatalerror\">No Hosts Found</font></p>";
                        }
                } //check for error state

        //if an error has occured then go back to the select!
        else {
                require ($conf['prefix'] . "/templates/show_bulk_select.inc");
        } //end else

} // action == view_bulk_update

/*!
	@function show_host_history
	@discussion  Show the host_history for the specified, host, ipint or hardware address.
	@param host_id		The ID of the host (referenced in DB)
	@param ip_int		The int value of the ip address
	@param ha		The hardware address of the machine in question
*/
elseif ($action == 'show_host_history') {
	$conf['ui']->show_host_history($host_id,$ip_int,$ha);
} // action == show_host_history

/*!
	@function really_delete_host
	@discussion This function also needs revision, we need to get away from the really stuff
*/
elseif ( $action == 'really_delete_host' ) {

        if ( $user->has_access($host->zone->id) )
                {
                $host->delete();
		$conf['ui']->display_confirmation("$host->name Deleted", "<b>The host, $host->name has been deleted succesfully.</b>", $conf['web_path'] . "/index.php"); 
                }
	else    {
		$conf['ui']->display_confirmation("Insufficant Access to $host->name", "<b>You do not have write access to $host->name, It was not deleted</b>", $conf['web_path'] . "/index.php"); 
		}	
} // action == really_delete_host

/*!
	@function show_children
	@discussion Show any child objects of said host, this checks for Hardware addresses, Cnames and MX records
	@param host_id	The ID (referenced in DB) of the host in question
*/
elseif ($action == "show_children") {
	$child = new Child("host",$host->id,0,$host);
	echo "<p align=\"center\"><font class=\"header1\">There are currently $child->total child objects of $host->name</font></p>";
	$child->show_children();
} // action == show_children

/*!
	@function show_hosts
	@discussion display multiple hosts, this uses the user->view object
	@param userview		Required for use, it will generate if one doesn't already exist
	@param sort_type	Used by View, the sort type Domain, IP, Hostname etc
	@param sort_order	ASC/DESC used by the View
*/
elseif ( $action == 'show_hosts' ) {
	if ( ! $user->view ) {
			$user->view = new View();
	} // if no view

	// this is needed because of stupid old browsers
	$conf['script'] = 'host.php';

	if ( $sort_type ) {
		$user->view->initialize($sort_type, $sort_offset);

		$hosts = $conf['zone']->hosts($user->view);
		$conf['ui']->show_hosts($hosts);
	} //if sort_type
	else {
		$user->view->initialize('name', $sort_offset);
		// if we pass the page a zone_id then let's switch the conf['zone'] to match
		if ($view_zone_id) 
			 { 
			 $conf['zone'] = new Zone($view_zone_id);
			 }
		$hosts = $conf['zone']->hosts($user->view);
		$conf['ui']->show_hosts($hosts);
	} // end else

} // action == show_hosts

/*!
	@function show_bulk_update
	@discussion displays the select for a bulk update
*/
elseif ( $action == "show_bulk_update") {
	$sql = "SELECT id FROM zone_range WHERE zone=" . $conf['zone']->id;

	if (!$user->view || $purge_view ) {
               	$user->view = new View($sql);
        }

        if ($sort_type) {
                $user->view->initialize($sort_type,$sort_offset);
        }
        else {
                $user->view->initialize('start', $sort_offset);
        }

        $ranges = get_all_ranges($user->view);
	$conf['script'] = "host.php";
	require ($conf['prefix'] . "/templates/show_ranges.inc"); 

} // action == show_bulk_update

/*!
	@function delete
	@discussion This is the base delete function, this should be confirm_delete... needs revision
*/
elseif ( $action == 'Delete' ) {
	//for gosh sakes man think of the children 
	$child = new Child("host",$host->id,0,$host);
	if (!($child->mxs AND $child->cnames)) 
	{ 
		$conf['ui']->show_confirm($conf['web_path'] . "/host.php?action=really_delete_host&amp;host_id=$host_id", 
		"Are you sure you want to delete the host $host->hostname?");
	} // if doesn't have the kiddies
	else    
	{
		echo "<font class=\"fatalerror\">The host $host->hostname has $child->total child objects are you sure you want to delete it?</font><br />";
		echo "<a href =\"" . $conf['web_path'] . "/host.php?action=really_delete_host&amp;host_id=$host_id\">[Delete]</a><br /><br />";
		$child->show_children();
	} // they've got kiddies!
} // action == delete

/*!
	@function select_bulk_hosts_zone_move
	@discussion Shows the select tool to get the hosts we want 
*/
elseif ($action == "select_bulk_hosts_zone_move") {

	require ($conf['prefix'] . "/templates/show_bulk_host_zone_move.inc");

} // select_bulk_hosts_zone_move

/*!
	@function view_bulk_hosts_zone_move
	@discussion After we have selected the hosts display them and let them pickt the ones they want to update
*/
elseif ($action == "view_bulk_hosts_zone_move") {
	
	// get the hosts we need
        $hosts = $host->bulk_get($select_type,$range_id,$start_range,$end_range,$domain_id,$hostname);


	require ($conf['prefix'] . "/templates/show_bulk_host_zone_move.inc");

} // view_bulk_hosts_zone_move

/*!
	@function confirm_move_host_zone
	@discussion Confirm that they really want to move the host between the two zones.
	@param $host_id	ID of the Host we are moving
	@param $zone_id ID of the zone we want to move it into
	@results Outputs a confirmation message, with a link pointing onto the actual move
*/
elseif ($action == "confirm_move_host_zone") {

	$zone = new Zone($zone_id);

	// the message we want to display
	$message = "Are you sure you want to move " . $host->hostname . " from " . $host->zone->name ." to " . $zone->name . "?";

	// show the confirm...
	$conf['ui']->show_confirm($conf['web_path'] . "/host.php?action=move_host_zone&host_id=$host_id&zone_id=$zone_id",$message);

} // confrim_move_host_zone

/*!
	@function confirm_move_hosts_zone
	@discussion Confirms that they really want to move all the hosts between the two zones.
	@param $hosts	Array of the Hosts we are moving
	@param $zone_id ID of the zone we want to move them into.
	@results Outputs a confirmation message, with a link pointing onto the actual move as well as a list of all the hosts selected
*/
elseif ($action == "confirm_move_hosts_zone") {
	
	$zone = new Zone($zone_id);

	// the message we want to display
	$message = "Are you sure you want to move the below hosts to " . $zone->name . "?<br /><br />";

	// setup the start of the table for the hosts we need to recurse through and display 
	$message .= "<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\">\n<tr class=\"widget-title\">\n\t<td colspan=\"2\">Selected Hosts</td>\n</tr>\n";

	foreach ($hosts as $host) {
		
		$host = new Host($host);

		$message .= "<tr>\n\t<td>" . $host->hostname . "</td>\n";
		$message .= "\t<td>" . $host->ip->ip_addr . "\n<input type=\"hidden\" name=\"hosts[]\" value=\"$host->id\" />\n";
		$mesasge .= "\t</td>\n</tr>";

	} // end foreach

	$message .= "\n</table>\n";

	$conf['ui']->show_confirm($conf['web_path'] . "/host.php?action=move_hosts_zone&zone_id=$zone_id",$message);

}

?>

<? show_template('footer'); ?>
